import os

import eero

from eero_adguard_sync.utils import app_paths
from eero_adguard_sync.models import EeroClientDevice, EeroNetworkDevice


class CookieStore(eero.SessionStorage):
    # See: https://github.com/343max/eero-client/blob/master/sample.py
    def __init__(self, cookie_file):
        self.cookie_file = os.path.abspath(cookie_file)

        try:
            with open(self.cookie_file, "r") as f:
                self.__cookie = f.read()
        except IOError:
            self.__cookie = None

    @property
    def cookie(self):
        return self.__cookie

    @cookie.setter
    def cookie(self, cookie):
        self.__cookie = cookie
        with open(self.cookie_file, "w+") as f:
            f.write(self.__cookie)


class EeroClient(eero.Eero):
    device_model_fields = {"ips", "mac", "nickname", "device_type"}
    eero_model_fields = {
        "mac_address",
        "ip_address",
        "model",
        "location",
        "gateway",
        "ipv6_addresses",
    }
    cookie_path = os.path.join(app_paths.app_data_path, "session.cookie")

    def __init__(self, cookie: str = None):
        session = CookieStore(self.cookie_path)
        if cookie:
            session.cookie = cookie
        super().__init__(session)

    @classmethod
    def clear_credentials(cls):
        try:
            os.remove(cls.cookie_path)
        except FileNotFoundError:
            pass

    def get_clients(self, network: str) -> list[EeroClientDevice]:
        devices: list[EeroClientDevice] = []
        for device in self.devices(network):
            new_device = {}
            for key in self.device_model_fields:
                new_device[key] = device[key]
            devices.append(EeroClientDevice(**new_device))
        for device in self.eeros(network):
            new_device = {}
            for key in self.eero_model_fields:
                new_device[key] = device[key]
            devices.append(EeroNetworkDevice(**new_device).as_client_device())
        return devices



from dataclasses import dataclass, field

from eero_adguard_sync.models.eero.client_device import EeroClientDevice


@dataclass
class EeroNetworkDevice:
    mac_address: str
    ip_address: str
    model: str
    location: str
    gateway: bool = False
    ipv6_addresses: list[dict] = field(default_factory=list)

    @property
    def ips(self) -> list[str]:
        ipv6_addresses = [i["address"] for i in self.ipv6_addresses if i.get("address")]
        return [self.ip_address, *ipv6_addresses]

    @property
    def nickname(self) -> str:
        nickname = f"{self.location} {self.model}"
        if self.gateway:
            nickname += " (Gateway)"
        return nickname

    def as_client_device(self) -> EeroClientDevice:
        return EeroClientDevice(
            ips=self.ips,
            nickname=self.nickname,
            mac=self.mac_address,
            device_type="generic",
        )



import click

from eero_adguard_sync import VERSION
from eero_adguard_sync.commands import sync, clear


@click.group()
@click.version_option(VERSION)
def cli():
    pass


cli.add_command(sync)
cli.add_command(clear)


if __name__ == "__main__":
    cli()



from .dhcp import DHCPClient, DHCPClientDevice, DHCPClientTableDiff, DHCPClientTable
from .adguard import AdGuardCredentialSet, AdGuardClientDevice
from .eero import EeroClientDevice, EeroNetworkDevice



from abc import ABC, abstractmethod, abstractclassmethod
from typing import Generic, TypeVar

from .client_table import DHCPClient

T = TypeVar("T")


class DHCPClientDevice(ABC, Generic[T]):
    @abstractmethod
    def to_dhcp_client(self) -> DHCPClient:
        raise NotImplementedError

    @classmethod
    @abstractmethod
    def from_dhcp_client(cls, dhcp_client: "DHCPClient") -> T:
        raise NotImplementedError
